#include "fxlib.h"
#include "scif.h"

int sprintf(char *buf, const char *fmt, ...);

volatile struct st_scif *SCIF;

// uncomment if you are using the unfinished two-bit-per-sample option in the converter
//#define TWO_BPS

// P_phi and BIT_RATE are probably wrong. However, their wrongness
// cancels out and it plays at the right speed. I suggest you
// don't change them.

#define P_phi 12000000 // Peripheral module operating frequency

#ifdef TWO_BPS
#define BIT_RATE (44100*8*4)
#else
#define BIT_RATE (44100*8)
#endif

extern const char sound_data[];
extern int sound_data_len;

void SetChannel(int i)
{
	SCIF = (i == 2 ? &SCIF2 : &SCIF0);

	SCIF->SCSCR.BIT.TE = 0; // disable transmit
	SCIF->SCSCR.BIT.RE = 0; // disable receive

	SCIF->SCFER.WORD = 0;

	SCIF->SCSSR.BIT.RDF = 0;
	SCIF->SCSSR.BIT.ER = SCIF->SCSSR.BIT.DR = SCIF->SCSSR.BIT.BRK = 0;

	SCIF->SCSMR.BIT.SRC = 1; // sampling rate 1/16 (?)
	SCIF->SCSMR.BIT.CA = 0; // 0=asynchronous mode 1=synchronous mode
	SCIF->SCSMR.BIT.CHR = 0; // 8-bit data (ignored in sychronous mode)
	SCIF->SCSMR.BIT._PE = 0; // parity disabled (ignored in sychronous mode)
	SCIF->SCSMR.BIT.OE = 0; // odd parity (though it's irrelevant as parity is disabled)
	SCIF->SCSMR.BIT.STOP = 0; // one stop bit (ignored in sychronous mode)
	SCIF->SCSMR.BIT.CKS = 0; // clock source is P_phi

	SCIF->SCSCR.BIT.TSIE = 0;
	SCIF->SCSCR.BIT.ERIE = 0;
	SCIF->SCSCR.BIT.BRIE = 0;
	SCIF->SCSCR.BIT.DRIE = 0;
	SCIF->SCSCR.BIT.TIE = 0;
	SCIF->SCSCR.BIT.RIE = 0;

	SCIF->SCFCR.BIT.TSE = 0; // "transmit stop function" disabled
	SCIF->SCFCR.BIT.MCE = 0; // modem control signals disabled
	SCIF->SCFCR.BIT.LOOP = 0; // loopback testing disabled

	SCIF->SCSCR.BIT.CKE = 1; // output internal clock

	// SCBRR = (P_phi / (4 * 2^(2*CKS-1) * BITRATE) * 10^6 - 1
	SCIF->SCBRR = (int)(((double)P_phi) / (4 * 2 * (double)BIT_RATE) * 1000000 - 1);

	for(i=0;i<8192;i++);

	SCIF->SCFCR.BIT.RTRG = 0;
	SCIF->SCFCR.BIT.TTRG = 32;
	SCIF->SCFCR.BIT.MCE = 0;
	SCIF->SCFCR.BIT.LOOP = 0;
	SCIF->SCFCR.BIT.RFRST = SCIF->SCFCR.BIT.TFRST = 0;
	SCIF->SCSCR.BIT.TE = SCIF->SCSCR.BIT.RE = 1;
	SCIF->SCSCR.BIT.TIE = SCIF->SCSCR.BIT.RIE = 0;
}

const char *pCurData, *pDataEnd;

unsigned char rle_count = 0, rle_val;

unsigned char rle_next()
{
	if((rle_count & 0x7F) == 0)
	{
		rle_count = *pCurData++;
		if(!(rle_count & 0x80))
			rle_val = *pCurData++;
	}
	if(rle_count & 0x80)
		rle_val = *pCurData++;
	--rle_count;
	return rle_val;
}

#ifdef TWO_BPS
unsigned char decode_table[4] = {1, 3, 7, 15};
unsigned char decode_bits(unsigned char in)
{
	return decode_table[in & 3] | (decode_table[in >> 2] << 4);
}
#endif

int AddIn_main(int isAppli, unsigned short OptionNum)
{
	unsigned char old_stbcr3 = STBCR3.BYTE;
	unsigned int key;
	char buf[30];
	unsigned int data;

	pCurData = sound_data;
	pDataEnd = sound_data + sound_data_len;

    Bdisp_AllClr_DDVRAM();

	STBCR3.BYTE = 0xFD; // SCIF2 enabled

	PFC.PGCR.BIT.PG6MD = 1; // port G6 = output (???)
	PG.DR.BYTE |= 0x60; // ???????

	SetChannel(2);                            

	locate(1,1); Print("CASIO music player   ");
	locate(1,2); Print("Created by immibis   ");
	locate(1,3); Print("Press [MENU] to exit.");
	Bdisp_PutDisp_DD();

    while(1)
	{
		while(SCIF->SCSSR.BIT.TDFE)
		{
#ifndef TWO_BPS
			SCIF->SCFTDR = rle_next();
#else
			data = rle_next();
			SCIF->SCFTDR = decode_bits(data & 15);
			SCIF->SCFTDR = decode_bits(data >> 4);
#endif
			
			if(pCurData >= pDataEnd)
				pCurData = sound_data;
			SCIF->SCSSR.BIT.TDFE = 0;
		}

		if(IsKeyDown(KEY_CTRL_MENU))
			goto finish;
    }

finish:
	SCIF->SCSCR.BIT.TE = 0; // disable transmit
	SCIF->SCSCR.BIT.RE = 0; // disable receive
	STBCR3.BYTE = old_stbcr3;
	
    return 1;
}






