#include <stdio.h>

// do not change
#define RLE

// This isn't actually bits-per-sample, it's 2^bits-per-sample.
// Supported values are 2 and 4. If 4, uncomment TWO_BPS in the player.
#define BPS 2

// if uncommented, creates a file out.wav containing the exact sound the
// calculator will play. Doesn't work properly - the resulting file sounds
// nothing like the calculator's output.
//#define WRITE_WAV

// uncomment if a chiptune created with Beepola
#define BEEPOLA

// uncomment if stereo
//#define STEREO

// must be 44kHz 8-bit WAV
#define INFILE "audio.wav"

// maximum output size. It may go up to 255 bytes over this.
#define SIZE_LIMIT 1200000

FILE *out, *wav_out;

int nbytes = 0;

void write_actual_byte(unsigned char ch)
{
	fprintf(out, "0x%2.2X,", ch);
	++nbytes;
	if(!(nbytes & 63))
		fprintf(out, "\n");
}

#ifdef RLE
int rle_count = 0, rle_val = 0;
unsigned char rle_lbuf[256];
int rle_lcount = 0;
void rle_write_literal()
{
	if(rle_lcount == 0)
		return;
	write_actual_byte(0x80 | rle_lcount);
	for(int k = 0; k < rle_lcount; k++)
		write_actual_byte(rle_lbuf[k]);
	rle_lcount = 0;
}
void rle_add_literal(unsigned char ch)
{
	rle_lbuf[rle_lcount++] = ch;
	if(rle_lcount == 127)
		rle_write_literal();
}
void write_finish()
{
	if(rle_count <= 2)
	{
		for(int k = 0; k < rle_count; k++)
			rle_add_literal(rle_val);
		return;
	}
	if(rle_lcount > 0)
		rle_write_literal();
	write_actual_byte(rle_count);
	write_actual_byte(rle_val);
}
void write_byte(unsigned char ch)
{
	if(rle_count == 0)
		rle_val = ch, rle_count = 1;
	else if(rle_val == ch && rle_count < 127)
		rle_count++;
	else
	{
		write_finish();
		rle_val = ch;
		rle_count = 1;
	}
}
#else // no compression
#define write_byte write_actual_byte
#define write_finish()
#endif

unsigned char bitbuf = 0;
int nbits = 0;
void write_bit_calc(int bit)
{
	if(bit) bitbuf |= 1 << nbits;
	++nbits;
	if(nbits == 8)
	{
		// simulate start and stop bits
		//putc(0xFF, wav_out);
		//putc(0x00, wav_out);
		
		write_byte(bitbuf);
		bitbuf = 0;
		nbits = 0;
	}
}

void write_bit_wav(int bit)
{
#ifdef WRITE_WAV
	putc(bit ? 0xFF : 0x00, wav_out);
#endif
}
void write_bit(int bit)
{
	write_bit_wav(bit);
	write_bit_calc(bit);
}


int main(int argc, char **argv)
{
	FILE *inf = fopen(INFILE, "rb");
	out = fopen("../out.c", "w");
	fseek(inf, 44, SEEK_SET);

#ifdef WRITE_WAV
	wav_out = fopen("out.wav", "wb");
	fwrite("RIFF\0\0\0\0WAVEfmt \x10\0\0\0\x01\0\x01\0\0\0\0\0\0\0\0\0\x01\0\x08\0data\0\0\0\0", 1, 44, wav_out);

	int samplerate = 44100*BPS;

	fseek(wav_out, 24, SEEK_SET);
	fwrite(&samplerate, 4, 1, wav_out);
	fwrite(&samplerate, 4, 1, wav_out);
	fseek(wav_out, 0, SEEK_END);
#endif

	int nsamples = 0;

#ifdef BEEPOLA
	int pulse = 1;
#else
	int total_error = 0;
	int prevbit = 0, runlength = 0;
#endif

	fprintf(out, "const char sound_data[] = {\n");

	while(true)
	{
		unsigned int in = getc(inf);
		if(in == (unsigned int)EOF)
			break;
		
		nsamples++;
		
#ifdef STEREO
		getc(inf);
#endif
		
		int bit;
#ifdef BEEPOLA
		if(in > 0xB0)
			write_bit(1);
		else if(in < 0x90)
			write_bit(0);
		else
			write_bit(pulse = !pulse);
#elif BPS == 2
		write_bit(in > 0x80);
#else
		{
			const int N = BPS;
			bit = (in / (255 / N + 1));
#if BPS == 4
			write_bit_calc(bit & 2);
			write_bit_calc(bit & 1);
#else
#error fixme
#endif
			for(int k = 0; k < bit; k++)
				write_bit_wav(1);
			for(int k = bit; k < N; k++)
				write_bit_wav(0);

			//write_bit(int(in) >= 0x80);
			
			//write_bit(int(in) >= 0x50);
			//write_bit(int(in) >= 0xA0);
		}
#endif

		//write_bit(bit);

		if(nbytes >= SIZE_LIMIT)
		{
			printf("hit size limit, stopping\n");
			break;
		}
	}

	write_finish();

	fprintf(out, "\n};\n");
	fprintf(out, "int sound_data_len = %i;\n", nbytes);

	printf("%i bytes\n", nbytes);
	printf("%i samples (%.2f seconds)\n", nsamples, nsamples/44100.0);

#ifdef WRITE_WAV
	int wavsize = ftell(wav_out);
	int temp;
	
	temp = wavsize;
	fseek(wav_out, 4, SEEK_SET);
	fwrite(&temp, 4, 1, wav_out);
	
	temp -= 44;
	fseek(wav_out, 40, SEEK_SET);
	fwrite(&temp, 4, 1, wav_out);

	fclose(wav_out);
#endif

	fclose(out);
	fclose(inf);

	return 0;
}